@@ -94,7 +94,7 @@ inline double giac_log(double d){
 
 #include <stdexcept>
 #include "help.h"
-#if !defined(GIAC_HAS_STO_38) // && !defined(ConnectivityKit) && !defined(BESTA_OS)
+#if 1 // !defined(GIAC_HAS_STO_38) // && !defined(ConnectivityKit) && !defined(BESTA_OS)
 #include "tinymt32.h"
 #endif
 
@@ -131,6 +131,8 @@ typedef unsigned char UTF8; /* typically 8 bits */
 typedef unsigned char Boolean; /* 0 or 1 */
 #endif
 
+  int system_no_deprecation(const char *command);
+
 /* Some fundamental constants */
 #define UNI_REPLACEMENT_CHAR (UTF32)0x0000FFFD
 #define UNI_MAX_BMP (UTF32)0x0000FFFF
@@ -177,8 +179,10 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   unsigned int utf8length(const wchar_t * wline);
 
 
-#if defined VISUALC || defined BESTA_OS
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
+#ifndef R_OK
   extern int R_OK;
+#endif
   int access(const char * ch,int mode);
   void usleep(int );
 #endif
@@ -213,6 +217,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   extern int history_begin_level;
 
   extern int debug_infolevel; // ==0 normal value
+  extern int printprog; // ==0 normal value, used to force program print at parse
   // >0 log some informations
   // <0 for internal use
   // ==-1:
@@ -227,10 +232,10 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   extern bool CAN_USE_LAPACK;
   extern bool simplify_sincosexp_pi;
 #ifndef RTOS_THREADX
-#ifndef BESTA_OS
+  //#ifndef BESTA_OS
   extern int CALL_LAPACK; // lapack is used if dim of matrix is >= CALL_LAPACK
   // can be changed using shell variable GIAC_LAPACK in icas
-#endif
+  //#endif
 #endif
   extern int FACTORIAL_SIZE_LIMIT;
   extern int GAMMA_LIMIT;
@@ -290,7 +295,17 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
 #ifdef TIMEOUT
   void control_c();
 #else
+#if 0
 #define control_c() if (ctrl_c) { interrupted = true; CERR << "Throwing exception for user interruption." << std::endl; throw(std::runtime_error("Stopped by user interruption.")); }
+#else
+#define control_c() if (ctrl_c) { \
+interrupted = true; \
+std::string source_path = __FILE__; \
+std::string source_filename = source_path.substr(source_path.find_last_of("/\\") + 1); \
+CERR << "Throwing exception for user interruption (" << source_filename << ":" << __LINE__ << ")" << std::endl; \
+throw(std::runtime_error("Stopped by user interruption.")); \
+}
+#endif
 #endif // TIMEOUT
 #endif // !NSPIRE, !FIR
 
@@ -527,10 +542,12 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
     bool _try_parse_i_;
     bool _specialtexprint_double_;
     bool _atan_tan_no_floor_;
+    bool _keep_acosh_asinh_;
     bool _keep_algext_;
+    int _python_compat_;
     int _angle_mode_;
     int _bounded_function_no_;
-    int _series_flags_; // bit1= full simplify, bit2=1 for truncation, bit3=atan does not rewrite sin/cos to tan, bit4=no back conversion, bit5=write<<1,1>> with series_variable_name, bit 6=write O() instead of order_size
+    int _series_flags_; // 1= full simplify, 2=1 for truncation, bit3=atan does not rewrite sin/cos to tan, bit4=no back conversion, bit5=write<<1,1>> with series_variable_name, bit 6=write O() instead of order_size, bit7= 1 diff in subst does not variable substitution
     int _step_infolevel_; 
     int _default_color_;
     double _epsilon_;
@@ -553,7 +570,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
     parser_lexer _pl;
     int _prog_eval_level_val ;
     int _eval_level;
-#if defined(GIAC_HAS_STO_38) || defined(ConnectivityKit)
+#if 0 // defined(GIAC_HAS_STO_38) || defined(ConnectivityKit)
     unsigned int _rand_seed;
 #else
     tinymt32_t _rand_seed;
@@ -571,6 +588,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
     std::string _autoname_;
     std::string _format_double_;
     std::string _autosimplify_;
+    std::string _lastprog_name_;
     std::vector<logo_turtle> _turtle_stack_; 
     double _total_time_;
     void * _evaled_table_;
@@ -633,6 +651,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   std::string autosimplify(GIAC_CONTEXT);
   std::string autosimplify(const std::string & s,GIAC_CONTEXT);
 
+  bool csv_guess(const char * data,int count,char & sep,char & nl,char & decsep);
   std::string & format_double(GIAC_CONTEXT);
 
   int check_thread(context * contextptr);
@@ -644,6 +663,10 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   int & xcas_mode(GIAC_CONTEXT);
   void xcas_mode(int b,GIAC_CONTEXT);
 
+  int & python_compat(GIAC_CONTEXT);
+  void python_compat(int b,GIAC_CONTEXT);
+  int array_start(GIAC_CONTEXT);
+
   int & calc_mode(GIAC_CONTEXT);
   int abs_calc_mode(GIAC_CONTEXT);
   void calc_mode(int b,GIAC_CONTEXT);
@@ -684,6 +707,9 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   bool & atan_tan_no_floor(GIAC_CONTEXT);
   void atan_tan_no_floor(bool b,GIAC_CONTEXT);
 
+  bool & keep_acosh_asinh(GIAC_CONTEXT);
+  void keep_acosh_asinh(bool b,GIAC_CONTEXT);
+
   bool & keep_algext(GIAC_CONTEXT);
   void keep_algext(bool b,GIAC_CONTEXT);
 
@@ -730,6 +756,9 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   bool & rpn_mode(GIAC_CONTEXT);
   void rpn_mode(bool b,GIAC_CONTEXT);
 
+  std::string lastprog_name(GIAC_CONTEXT);
+  std::string lastprog_name(const std::string & b,GIAC_CONTEXT);
+
   logo_turtle & turtle(GIAC_CONTEXT);
   std::vector<logo_turtle> & turtle_stack(GIAC_CONTEXT);
 
@@ -797,7 +826,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   // void eval_level(int b,GIAC_CONTEXT);
   thread_param * thread_param_ptr(const context * contextptr);
 
-#if defined(GIAC_HAS_STO_38) || defined(ConnectivityKit)
+#if 0 // defined(GIAC_HAS_STO_38) || defined(ConnectivityKit)
   unsigned int & rand_seed(GIAC_CONTEXT);
 #else
   tinymt32_t * rand_seed(GIAC_CONTEXT);
@@ -900,6 +929,8 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   int equalposcomp(const std::vector<int> v,int i);
   int equalposcomp(const std::vector<short int> v,int i);
   int equalposcomp(int tab[],int f);
+  // attempt to convert Python-like programming structures to Xcas
+  std::string python2xcas(const std::string & s_orig,GIAC_CONTEXT);
   std::string find_doc_prefix(int i);
   std::string find_lang_prefix(int i);
   int string2lang(const std::string & s); // convert "fr" to 1, "es" to 3 etc.
